<?php

namespace B2buy\Master\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use B2buy\Master\Forms\Masters as MastersForm;
use B2buy\Master\Http\Requests\MastersResourceRequest;
use B2buy\Master\Http\Resources\MastersResource;
use B2buy\Master\Http\Resources\MastersCollection;
use B2buy\Master\Models\Masters;
use B2buy\Master\Actions\MastersAction;
use B2buy\Master\Actions\MastersActions;


/**
 * Resource controller class for masters.
 */
class MastersResourceController extends BaseController
{

    /**
     * Initialize masters resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = MastersForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('b2buy.master.modules'), 'master', guard_url('master'));
            return $next($request);
        });
    }

    /**
     * Display a list of masters.
     *
     * @return Response
     */
    public function index(MastersResourceRequest $request)
    {
        $request = $request->all();
        $page = MastersActions::run('paginate', $request);

        $data = new MastersCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('master::masters.names'))
            ->view('master::masters.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display masters.
     *
     * @param Request $request
     * @param Model   $masters
     *
     * @return Response
     */
    public function show(MastersResourceRequest $request, Masters $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MastersResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('master::masters.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('master::masters.show')
            ->output();
    }

    /**
     * Show the form for creating a new masters.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MastersResourceRequest $request, Masters $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MastersResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('master::masters.name'))
            ->view('master::masters.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new masters.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MastersResourceRequest $request, Masters $model)
    {
        try {
            $request = $request->all();
            $model = MastersAction::run('store', $model, $request);
            $data = new MastersResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('master::masters.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('master/masters/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/master/masters'))
                ->redirect();
        }

    }

    /**
     * Show masters for editing.
     *
     * @param Request $request
     * @param Model   $masters
     *
     * @return Response
     */
    public function edit(MastersResourceRequest $request, Masters $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MastersResource($model);
        // return view('master::masters.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('master::masters.name'))
            ->view('master::masters.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the masters.
     *
     * @param Request $request
     * @param Model   $masters
     *
     * @return Response
     */
    public function update(MastersResourceRequest $request, Masters $model)
    {
        try {
            $request = $request->all();
            $model = MastersAction::run('update', $model, $request);
            $data = new MastersResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::masters.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('master/masters/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/masters/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the masters.
     *
     * @param Model   $masters
     *
     * @return Response
     */
    public function destroy(MastersResourceRequest $request, Masters $model)
    {
        try {

            $request = $request->all();
            $model = MastersAction::run('destroy', $model, $request);
            $data = new MastersResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('master::masters.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('master/masters/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/masters/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
