<?php

namespace B2buy\Master\Http\Controllers;

use Exception;
use B2buy\Master\Actions\MastersWorkflow;
use B2buy\Master\Http\Requests\MastersWorkflowRequest;
use B2buy\Master\Http\Resources\MastersResource;
use B2buy\Master\Models\Masters;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class MastersWorkflowController extends BaseController
{
    /**
     * Action controller function for masters.
     *
     * @param Model $masters
     * @param action next action for the masters.
     *
     * @return Response
     */
    public function __invoke(MastersWorkflowRequest $request, Masters $masters, $trasition)
    {
        try {
            $request = $request->all();
            $masters = MastersWorkflow::run($trasition, $masters, $request);
            $data = new MastersResource($masters);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('master::masters.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('master/masters/' . $masters->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('master/masters/' . $masters->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the masters and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $masters
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $masters, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $masters = Masters::findBySignedId($masters);
        $transitions = $this->transitions($masters, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('master::masters.name') . ' ' . trans('Approvals'))
            ->view('master::public.masters.workflow')
            ->layout('mobile')
            ->data(compact('masters', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the masters.
     *
     * @param PublicRequest $request
     * @param string $masters
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $masters, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $masters = Masters::findBySignedId($masters);
        $user = user();
        $masters = MastersWorkflow::run($transition, $masters, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($masters, $user)
    {
        $transitions = [];
        foreach ($masters->workflow()->transitions($masters) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.masters.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'masters' => $masters->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $masters->workflow()->form($value);
            $array['label'] = trans('master::masters.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
